@@ -1,6 +1,6 @@
 # This script generates the PyQt configuration and generates the Makefiles.
 #
-# Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of PyQt.
 # 
@@ -16,13 +16,8 @@
 # GPL Exception version 1.1, which can be found in the file
 # GPL_EXCEPTION.txt in this package.
 # 
-# Please review the following information to ensure GNU General
-# Public Licensing requirements will be met:
-# http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-# you are unsure which license is appropriate for your use, please
-# review the following information:
-# http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-# or contact the sales department at sales@riverbankcomputing.com.
+# If you are unsure which license is appropriate for your use, please
+# contact the sales department at sales@riverbankcomputing.com.
 # 
 # This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 # WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -38,10 +33,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040801
-pyqt_version_str = "4.8.1"
+pyqt_version = 0x040901
+pyqt_version_str = "4.9.1"
 
-sip_min_version = 0x040b02
+sip_min_version = 0x040d01
 
 qt_version = 0
 qt_edition = ""
@@ -71,8 +66,8 @@ dbuslibdirs = []
 dbuslibs = []
 
 
-# Under Windows qmake and the Qt DLLs must be into the system PATH otherwise
-# the dynamic linker won't be able to resolve the symbols.  On other systems we
+# Under Windows qmake and the Qt DLLs must be on the system PATH otherwise the
+# dynamic linker won't be able to resolve the symbols.  On other systems we
 # assume we can just run qmake by using its full pathname.
 if sys.platform == 'win32':
     MSG_CHECK_QMAKE = "Make sure you have a working Qt v4 qmake on your PATH."
@@ -175,9 +170,12 @@ def create_optparser():
     g.add_option("--assume-shared", action="store_true", default=False,
             dest="assume_shared", help="assume that the Qt libraries have "
             "been built as shared libraries [default: check]")
+    g.add_option("-T", "--no-timestamp", action="store_true", default=False,
+            dest="no_timestamp", help="suppress timestamps in the header "
+            "comments of generated code [default: include timestamps]")
 
     if sys.platform != 'win32':
-        if sys.platform in ('linux2', 'darwin'):
+        if sys.platform in ('linux2', 'linux3', 'darwin'):
             pip_default = True
             pip_default_str = "enabled"
         else:
@@ -280,7 +278,8 @@ class pyrccMakefile(sipconfig.ProgramMakefile):
                 build_file=os.path.join(src_dir, "pyrcc", "pyrcc.sbf"),
                 dir="pyrcc", install_dir=opts.pyqtbindir, console=1,
                 qt=["QtCore", "QtXml"], debug=opts.debug, warnings=1,
-                universal=sipcfg.universal, arch=sipcfg.arch)
+                universal=sipcfg.universal, arch=sipcfg.arch,
+                deployment_target=sipcfg.deployment_target)
 
     def generate_target_default(self, mfile):
         """Generate the default target."""
@@ -333,7 +332,10 @@ class ConfigurePyQt4:
             0x040602: "Qt_4_6_1",
             0x040603: "Qt_4_6_2",
             0x040700: "Qt_4_6_3",
-            0x050000: "Qt_4_7_0"
+            0x040701: "Qt_4_7_0",
+            0x040702: "Qt_4_7_1",
+            0x040800: "Qt_4_7_2",
+            0x050000: "Qt_4_8_0"
         }
 
     def check_modules(self):
@@ -358,6 +360,8 @@ class ConfigurePyQt4:
         check_module("QtMultimedia", "QAudioDeviceInfo",
                 "new QAudioDeviceInfo()")
         check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
+        check_module("QtDBus", "qdbusconnection.h",
+                "QDBusConnection::systemBus()")
         check_module("QtDeclarative", "qdeclarativeview.h",
                 "new QDeclarativeView()")
         check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
@@ -460,9 +464,33 @@ class ConfigurePyQt4:
         if "QtNetwork" in pyqt_modules:
             generate_code("QtNetwork")
 
+        if "QtDBus" in pyqt_modules:
+            qpy_inc_dir, qpy_lib_dir, qpy_lib = self._qpy_directories("QtDBus", "qpydbus")
+
+            if opts.bigqt:
+                cons_xtra_incdirs.append(qpy_inc_dir)
+                cons_xtra_libdirs.append(qpy_lib_dir)
+                cons_xtra_libs.append(qpy_lib)
+
+                generate_code("QtDBus")
+            else:
+                generate_code("QtDBus", extra_include_dirs=[qpy_inc_dir],
+                        extra_lib_dirs=[qpy_lib_dir], extra_libs=[qpy_lib])
+
         if "QtOpenGL" in pyqt_modules:
             generate_OpenGL_extras()
-            generate_code("QtOpenGL")
+
+            qpy_inc_dir, qpy_lib_dir, qpy_lib = self._qpy_directories("QtOpenGL", "qpyopengl")
+
+            if opts.bigqt:
+                cons_xtra_incdirs.append(qpy_inc_dir)
+                cons_xtra_libdirs.append(qpy_lib_dir)
+                cons_xtra_libs.append(qpy_lib)
+
+                generate_code("QtOpenGL")
+            else:
+                generate_code("QtOpenGL", extra_include_dirs=[qpy_inc_dir],
+                        extra_lib_dirs=[qpy_lib_dir], extra_libs=[qpy_lib])
 
         if "QtScript" in pyqt_modules:
             generate_code("QtScript")
@@ -649,6 +677,9 @@ class ConfigurePyQt4:
         else:
             qpy_lib_dir = qpy_dir
 
+            if sys.platform == 'darwin' and opts.debug:
+                lib_name += '_debug'
+
         return os.path.join(src_dir, qpy_dir), os.path.abspath(qpy_lib_dir), lib_name
 
     def _static_plugins(self, mname):
@@ -732,12 +763,18 @@ class ConfigurePyQt4:
         if "QtGui" in pyqt_modules:
             qpylibs["QtGui"] = "qpygui.pro"
 
+        if "QtDBus" in pyqt_modules:
+            qpylibs["QtDBus"] = "qpydbus.pro"
+
         if "QtDeclarative" in pyqt_modules:
             qpylibs["QtDeclarative"] = "qpydeclarative.pro"
 
         if "QtDesigner" in pyqt_modules:
             qpylibs["QtDesigner"] = "qpydesigner.pro"
 
+        if "QtOpenGL" in pyqt_modules:
+            qpylibs["QtOpenGL"] = "qpyopengl.pro"
+
         # Run qmake to generate the Makefiles.
         qmake_args = fix_qmake_args()
         cwd = os.getcwd()
@@ -759,8 +796,11 @@ class ConfigurePyQt4:
             if sipcfg.universal:
                 f.write("QMAKE_MAC_SDK = %s\n" % sipcfg.universal)
 
+            if sipcfg.deployment_target:
+                f.write("QMAKE_MACOSX_DEPLOYMENT_TARGET = %s\n" % sipcfg.deployment_target)
+
             inc_path = [sipcfg.py_inc_dir]
-            if qpy in ("QtCore", "QtDeclarative"):
+            if qpy in ("QtCore", "QtDBus", "QtDeclarative", "QtOpenGL"):
                 if sipcfg.sip_inc_dir != sipcfg.py_inc_dir:
                     inc_path.insert(0, sipcfg.sip_inc_dir)
 
@@ -821,7 +861,8 @@ include(%s)
                 qt=["QtCore"],
                 debug=opts.debug,
                 universal=sipcfg.universal,
-                arch=sipcfg.arch
+                arch=sipcfg.arch,
+                deployment_target=sipcfg.deployment_target
             )
 
             add_makefile_extras(makefile, dbusincdirs, dbuslibdirs, dbuslibs)
@@ -854,6 +895,13 @@ include(%s)
             gui = False
             use_arch = ''
 
+        # The pyuic directory may not exist if we are building away from the
+        # source directory.
+        try:
+            os.mkdir("pyuic")
+        except OSError:
+            pass
+
         uicdir=os.path.join(pyqt_modroot, "uic")
         wrapper = sipconfig.create_wrapper(os.path.join(uicdir, "pyuic.py"), os.path.join("pyuic", "pyuic4"), gui, use_arch)
 
@@ -879,11 +927,12 @@ include(%s)
                 dir="pylupdate",
                 install_dir=opts.pyqtbindir,
                 console=1,
-                qt=["QtCore", "QtGui", "QtXml"],
+                qt=["QtCore", "QtXml"],
                 debug=opts.debug,
                 warnings=1,
                 universal=sipcfg.universal,
-                arch=sipcfg.arch
+                arch=sipcfg.arch,
+                deployment_target=sipcfg.deployment_target
             )
 
             makefile.extra_include_dirs.append(os.path.join(src_dir, "pylupdate"))
@@ -902,10 +951,13 @@ include(%s)
             py_major = sipcfg.py_version >> 16
             py_minor = (sipcfg.py_version >> 8) & 0x0ff
 
+            abi = getattr(sys, 'abiflags', '')
+
             if sys.platform == 'win32':
+                # Use abiflags in case it is supported in a future version.
                 lib_dir_flag = quote("-L%s" % sipcfg.py_lib_dir)
-                link = "%s -lpython%d%d" % (lib_dir_flag, py_major, py_minor)
-                pysh_lib = "python%d%d.dll" % (py_major, py_minor)
+                link = "%s -lpython%d%d%s" % (lib_dir_flag, py_major, py_minor, abi)
+                pysh_lib = "python%d%d%s.dll" % (py_major, py_minor, abi)
             else:
                 # Use distutils to get the additional configuration.
                 from distutils.sysconfig import get_config_vars
@@ -915,15 +967,21 @@ include(%s)
                     # We need to work out how to specify the right framework
                     # version.
                     link = "-framework Python"
-                elif ("--enable-shared" in ducfg.get("CONFIG_ARGS", "") and
-                      glob.glob("%s/lib/libpython%d.%d*" % (ducfg["exec_prefix"], py_major, py_minor))):
-                    lib_dir_flag = quote("-L%s/lib" % ducfg["exec_prefix"])
-                    link = "%s -lpython%d.%d" % (lib_dir_flag, py_major, py_minor)
+                elif "--enable-shared" in ducfg.get("CONFIG_ARGS", ""):
+                    if glob.glob("%s/lib/libpython%d.%d*" % (ducfg["exec_prefix"], py_major, py_minor)):
+                        lib_dir_flag = quote("-L%s/lib" % ducfg["exec_prefix"])
+                    elif glob.glob("%s/libpython%d.%d*" % (ducfg["LIBDIR"], py_major, py_minor)):
+                        lib_dir_flag = quote("-L%s" % ducfg["LIBDIR"])
+                    else:
+                        sipconfig.inform("Qt Designer plugin disabled because Python library couldn't be found")
+                        opts.designer_plugin = False
+
+                    link = "%s -lpython%d.%d%s" % (lib_dir_flag, py_major, py_minor, abi)
                 else:
                     sipconfig.inform("Qt Designer plugin disabled because Python library is static")
                     opts.designer_plugin = False
 
-                pysh_lib = ducfg["LDLIBRARY"]
+                pysh_lib = ducfg.get("LDLIBRARY", "")
 
             if opts.designer_plugin:
                 sipconfig.inform("Creating Qt Designer plugin Makefile...")
@@ -954,6 +1012,9 @@ include(%s)
                 if sipcfg.universal:
                     fout.write("QMAKE_MAC_SDK = %s\n" % sipcfg.universal)
 
+                if sipcfg.deployment_target:
+                    fout.write("QMAKE_MACOSX_DEPLOYMENT_TARGET = %s\n" % sipcfg.deployment_target)
+
                 if src_dir != os.path.curdir:
                     fout.write("VPATH = %s\n" % os.path.join(src_dir, "designer"))
 
@@ -1078,7 +1139,7 @@ def create_config(module, template, macros):
     sipconfig.create_config_module(module, template, content, macros)
 
 
-def run_command(cmd):
+def run_command(cmd, envvars=None):
     """Run a command and display the output if verbose mode is enabled.
 
     cmd is the command to run.
@@ -1086,7 +1147,7 @@ def run_command(cmd):
     if opts.verbose:
         sys.stdout.write(cmd + "\n")
 
-    fout = get_command_stdout(cmd, and_stderr=True)
+    fout = get_command_stdout(cmd, and_stderr=True, envvars=envvars)
 
     # Read stdout and stderr until there is no more output.
     lout = fout.readline()
@@ -1204,15 +1265,21 @@ def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None
         qt.append("QtCore")
 
     makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=qt, warnings=0,
-            opengl=opengl, debug=opts.debug, arch=sipcfg.arch)
+            opengl=opengl, debug=opts.debug, arch=sipcfg.arch,
+            deployment_target=sipcfg.deployment_target)
 
     add_makefile_extras(makefile, extra_include_dirs, extra_lib_dirs, extra_libs)
 
     exe, build = makefile.build_command(name)
 
+    if sipcfg.deployment_target:
+        envvars = {'MACOSX_DEPLOYMENT_TARGET': '%s' % sipcfg.deployment_target}
+    else:
+        envvars = None
+
     # Make sure the executable file doesn't exist.
     remove_file(exe)
-    run_command(build)
+    run_command(build, envvars=envvars)
 
     if not os.access(exe, os.X_OK):
         return None
@@ -1265,9 +1332,15 @@ def check_vendorid():
                              "correct directory." % opts.vendincdir)
 
 
-def get_command_stdout(cmd, and_stderr=False):
+def get_command_stdout(cmd, and_stderr=False, envvars=None):
     """Return stdout (and optionally stderr) from the given command.
     """
+    if envvars is not None:
+        env = os.environ.copy()
+        env.update(envvars)
+    else:
+        env = None
+
     try:
         import subprocess
     except ImportError:
@@ -1284,7 +1357,7 @@ def get_command_stdout(cmd, and_stderr=False):
         stderr = subprocess.PIPE
 
     p = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE, stderr=stderr)
+            stdout=subprocess.PIPE, stderr=stderr, env=env)
 
     return p.stdout
 
@@ -1301,8 +1374,10 @@ def check_dbus():
         sipconfig.inform("DBus v1 does not seem to be installed.")
         return
 
-    # Using str() means it will work with both Python v2 and v3.
-    for f in str(iflags).split():
+    if sys.hexversion >= 0x03000000:
+        iflags = iflags.decode()
+
+    for f in iflags.split():
         if f.startswith("-I"):
             dbusincdirs.append(f[2:])
         elif f.startswith("-L"):
@@ -1462,6 +1537,7 @@ def needed_qt_libs(mname, qt_libs):
     # as well as the libraries.
     LIB_DEPS = {
         "QtCore": [],
+        "QtDBus": ["QtCore"],
         "QtDeclarative": ["QtNetwork", "QtGui"],
         "QtGui": ["QtCore"],
         "QtHelp": ["QtGui"],
@@ -1547,8 +1623,11 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
             qt_libs = []
             needed_qt_libs(mname, qt_libs)
 
-    # Build the SIP command line.  Keyword argument support is enabled.
-    argv = ['"' + sipcfg.sip_bin + '"', '-k']
+    # Build the SIP command line.
+    argv = ['"' + sipcfg.sip_bin + '"', '-w']
+
+    if opts.no_timestamp:
+        argv.append("-T")
 
     if not opts.no_docstrings:
         argv.append("-o");
@@ -1573,8 +1652,9 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
         argv.append("-a")
         argv.append(mname + ".api")
 
+    # Pass the absolute pathname so that #line files are absolute.
     argv.append("-c")
-    argv.append(mname)
+    argv.append(os.path.abspath(mname))
 
     buildfile = os.path.join(mname, mname + ".sbf")
     argv.append("-b")
@@ -1640,7 +1720,8 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
         debug=opts.debug,
         universal=sipcfg.universal,
         arch=sipcfg.arch,
-        prot_is_public=opts.prot_is_public
+        prot_is_public=opts.prot_is_public,
+        deployment_target=sipcfg.deployment_target
     )
 
     add_makefile_extras(makefile, extra_include_dirs, extra_lib_dirs, extra_libs)
@@ -1648,6 +1729,39 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
     makefile.generate()
 
 
+def fix_license(name):
+    """ Fix the license file, if there is one, so that it conforms to the SIP
+    v5 syntax.
+    """
+
+    try:
+        f = open(name, "r")
+    except IOError:
+        sipconfig.error("Failed to open license file %s." % name)
+
+    f5 = open(name + "5", "w")
+
+    for line in f:
+        if line.startswith("%License"):
+            anno_start = line.find("/")
+            anno_end = line.rfind("/")
+
+            if anno_start < 0 or anno_end < 0 or anno_start == anno_end:
+                sipconfig.error("%s has missing annotations." % name)
+
+            annos = line[anno_start + 1:anno_end].split(", ")
+            annos5 = [anno[0].lower() + anno[1:] for anno in annos]
+
+            f5.write("%License(")
+            f5.write(", ".join(annos5))
+            f5.write(")\n")
+        else:
+            f5.write(line)
+
+    f5.close()
+    f.close()
+
+
 def check_license():
     """Handle the validation of the PyQt license.
     """
@@ -1666,7 +1780,7 @@ def check_license():
     if ltype is None:
         ltype = "GPL"
         lname = "GNU General Public License"
-        lfile = None
+        lfile = "pyqt-gpl.sip"
 
     sipconfig.inform("This is the %s version of PyQt %s (licensed under the %s) for Python %s on %s." % (ltype, pyqt_version_str, lname, sys.version.split()[0], sys.platform))
 
@@ -1720,12 +1834,14 @@ Type 'no' to decline the terms of the license.
                 if resp == "l":
                     os.system("more LICENSE")
 
-    # If there should be a license file then check it is where it should be.
-    if lfile:
-        if os.access(os.path.join(src_dir, "sip", lfile), os.F_OK):
-            sipconfig.inform("Found the license file %s." % lfile)
-        else:
-            sipconfig.error("Please copy the license file %s to the sip directory." % lfile)
+    # Check that the license file exists and fix its syntax.
+    lfile_path = os.path.join(src_dir, "sip", lfile)
+
+    if os.access(lfile_path, os.F_OK):
+        sipconfig.inform("Found the license file %s." % lfile)
+        fix_license(lfile_path)
+    else:
+        sipconfig.error("Please copy the license file %s to the sip directory." % lfile)
 
 
 def get_build_macros(overrides):
@@ -1833,8 +1949,12 @@ def get_qt_configuration():
     qmake_args = fix_qmake_args("-o " + make_file)
 
     if sys.platform == 'win32':
-        exe_file = os.path.join("release", app + ".exe")
-        make_target = " release"
+        if opts.debug:
+            exe_file = os.path.join("debug", app + ".exe")
+            make_target = " debug"
+        else:
+            exe_file = os.path.join("release", app + ".exe")
+            make_target = " release"
     elif sys.platform == "darwin":
         exe_file = os.path.join(app + ".app", "Contents", "MacOS", app)
     else:
@@ -1861,12 +1981,14 @@ SOURCES = %s
     f = open(cpp_file, "w")
 
     f.write(
-"""#include <QFile>
+"""#include <QCoreApplication>
+#include <QFile>
 #include <QLibraryInfo>
 #include <QTextStream>
 
-int main(int, char **)
+int main(int argc, char **argv)
 {
+    QCoreApplication app(argc, argv);
     QFile outf("%s");
 
     if (!outf.open(QIODevice::WriteOnly|QIODevice::Truncate|QIODevice::Text))
