@@ -2,7 +2,7 @@
  * This is the Qt Designer plugin that collects all the Python plugins it can
  * find as a widget collection to Designer.
  *
- * Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
  * 
  * This file is part of PyQt.
  * 
@@ -18,13 +18,8 @@
  * GPL Exception version 1.1, which can be found in the file
  * GPL_EXCEPTION.txt in this package.
  * 
- * Please review the following information to ensure GNU General
- * Public Licensing requirements will be met:
- * http://trolltech.com/products/qt/licenses/licensing/opensource/. If
- * you are unsure which license is appropriate for your use, please
- * review the following information:
- * http://trolltech.com/products/qt/licenses/licensing/licensingoverview
- * or contact the sales department at sales@riverbankcomputing.com.
+ * If you are unsure which license is appropriate for your use, please
+ * contact the sales department at sales@riverbankcomputing.com.
  * 
  * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
  * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -104,8 +99,9 @@ PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent)
     {
         QString dir = dirs.at(i);
 
-        // Get a list of all candidate plugin modules.
-        QStringList candidates = QDir(dir).entryList(QDir::Files);
+        // Get a list of all candidate plugin modules.  We sort by name to
+        // provide control over the order they are imported.
+        QStringList candidates = QDir(dir).entryList(QDir::Files, QDir::Name);
         QStringList plugins;
 
         for (int p = 0; p < candidates.size(); ++p)
@@ -165,15 +161,6 @@ PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent)
                 return;
         }
 
-        // Make sure we have sip.unwrapinstance.
-        if (!qtdesigner_custom)
-        {
-            qtdesigner_custom = getModuleAttr("PyQt4.QtDesigner", "QPyDesignerCustomWidgetPlugin");
-
-            if (!qtdesigner_custom)
-                return;
-        }
-
         // Convert the directory to a Python object with native separators.
 #if QT_VERSION >= 0x040200
         dir = QDir::toNativeSeparators(dir);
@@ -255,6 +242,17 @@ PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent)
                 continue;
             }
 
+            // Make sure we have QPyDesignerCustomWidgetPlugin.  We make sure
+            // this is after the import of the first plugin to allow that
+            // plugin to change any API versions.
+            if (!qtdesigner_custom)
+            {
+                qtdesigner_custom = getModuleAttr("PyQt4.QtDesigner", "QPyDesignerCustomWidgetPlugin");
+
+                if (!qtdesigner_custom)
+                    return;
+            }
+
             // Go through the module looking for types that implement
             // QDesignerCustomWidgetInterface (ie. by deriving from
             // QPyDesignerCustomWidgetPlugin).
