@@ -156,8 +156,7 @@ class MenuManager(QtCore.QObject):
         elif userCode == MenuManager.DOCUMENTATION:
             self.showDocInAssistant(self.currentInfo)
         elif userCode == MenuManager.QUIT:
-            self.window.loop = False
-            QtCore.QCoreApplication.quit()
+            self.window.quit()
         elif userCode == MenuManager.FULLSCREEN:
             self.window.toggleFullscreen()
         elif userCode == MenuManager.ROOT:
