@@ -1,6 +1,6 @@
 // This is the implementation of the Chimera class.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,13 +26,13 @@
 #include <Python.h>
 
 #include <QByteArray>
-#include <QList>
 #include <QMetaObject>
 #include <QMetaType>
 
 #include "qpycore_chimera.h"
 #include "qpycore_misc.h"
 #include "qpycore_pyqtpyobject.h"
+#include "qpycore_qpynullvariant.h"
 #include "qpycore_sip.h"
 #include "qpycore_types.h"
 
@@ -45,10 +40,18 @@
 // The registered int types.
 QList<QByteArray> Chimera::_registered_int_types;
 
-
 // The cache of previously parsed argument type lists.
 QHash<QByteArray, QList<const Chimera *> > Chimera::_previously_parsed;
 
+// The registered QVariant to PyObject convertors.
+Chimera::ToPyObjectConvertors Chimera::_registered_PyObject_convertors;
+
+// The registered PyObject to QVariant convertors.
+Chimera::ToQVariantConvertors Chimera::_registered_QVariant_convertors;
+
+// The registered PyObject to QVariant data convertors.
+Chimera::ToQVariantDataConvertors Chimera::_registered_QVariant_data_convertors;
+
 
 // Construct an invalid type.
 Chimera::Chimera()
@@ -90,6 +93,27 @@ void Chimera::registerIntType(const char *name)
 }
 
 
+// Register a QVariant to PyObject convertor.
+void Chimera::registerToPyObject(ToPyObjectFn fn)
+{
+    _registered_PyObject_convertors.append(fn);
+}
+
+
+// Register a PyObject to QVariant convertor.
+void Chimera::registerToQVariant(ToQVariantFn fn)
+{
+    _registered_QVariant_convertors.append(fn);
+}
+
+
+// Register a PyObject to QVariant data convertor.
+void Chimera::registerToQVariantData(ToQVariantDataFn fn)
+{
+    _registered_QVariant_data_convertors.append(fn);
+}
+
+
 // Parse an object as a type.
 const Chimera *Chimera::parse(PyObject *obj)
 {
@@ -131,6 +155,21 @@ const Chimera *Chimera::parse(PyObject *obj)
 }
 
 
+// Parse a C++ type name as a type.
+const Chimera *Chimera::parse(const QByteArray &name)
+{
+    Chimera *ct = new Chimera;
+
+    if (!ct->parse_cpp_type(name))
+    {
+        delete ct;
+        return 0;
+    }
+
+    return ct;
+}
+
+
 // Parse a meta-property as a type.
 const Chimera *Chimera::parse(const QMetaProperty &mprop)
 {
@@ -332,9 +371,6 @@ void Chimera::raiseParseException(const char *type, const char *context)
 // Parse the given Python type object.
 bool Chimera::parse_py_type(PyTypeObject *type_obj)
 {
-    _py_type = (PyObject *)type_obj;
-    Py_INCREF(_py_type);
-
     const sipTypeDef *td = sipTypeFromPyTypeObject(type_obj);
 
     if (td)
@@ -354,6 +390,8 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
         }
         else
         {
+            bool is_a_QObject = PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QObject));
+
             // If there is no assignment helper then assume it is a
             // pointer-type.
             if (!get_assign_helper())
@@ -361,6 +399,51 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
 
             _metatype = QMetaType::type(_name.constData());
 
+            // This will deal with cases where the registered type is a
+            // super-class and specifically allows for multiple inheritance.
+            // The problem we are solving is that we want a QGraphicsWidget to
+            // be handled as a QGraphicsItem (which Qt registers) rather than a
+            // QObject, specifically in the value passed as a QVariant to
+            // QGraphicsItem::itemChange().  This solution means that it will
+            // always be passed as a QGraphicsItem, even if there are
+            // circumstances where it should be passed as a QObject.  If we
+            // come across such a circumstance then we may need to remove this
+            // and implement a more specific solution in %MethodCode for the
+            // various itemChange() implementations.
+            if (_metatype == 0 && is_a_QObject)
+            {
+                PyObject *mro = type_obj->tp_mro;
+
+                Q_ASSERT(PyTuple_Check(mro));
+                Q_ASSERT(PyTuple_GET_SIZE(mro) >= 3);
+
+                for (SIP_SSIZE_T i = 1; i < PyTuple_GET_SIZE(mro) - 1; ++i)
+                {
+                    PyTypeObject *sc = (PyTypeObject *)PyTuple_GET_ITEM(mro, i);
+
+                    if (sc == sipSimpleWrapper_Type || sc == sipWrapper_Type)
+                        continue;
+
+                    QByteArray sc_name(sc->tp_name);
+
+                    // QObjects are always pointers.
+                    sc_name.append('*');
+
+                    _metatype = QMetaType::type(sc_name.constData());
+
+                    if (_metatype >= QMetaType::User)
+                    {
+                        _type = sipTypeFromPyTypeObject(sc);
+                        _name = sc_name;
+
+                        _py_type = (PyObject *)sc;
+                        Py_INCREF(_py_type);
+
+                        return true;
+                    }
+                }
+            }
+
             // If it is a user type then it must be a type that SIP knows
             // about but was registered by Qt.
             if (_metatype < QMetaType::User)
@@ -369,7 +452,7 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
                 {
                     _metatype = QMetaType::QWidgetStar;
                 }
-                else if (PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QObject)))
+                else if (is_a_QObject)
                 {
                     _metatype = QMetaType::QObjectStar;
                 }
@@ -438,6 +521,9 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
     if (_name.isEmpty())
         _name = QMetaType::typeName(_metatype);
 
+    _py_type = (PyObject *)type_obj;
+    Py_INCREF(_py_type);
+
     return true;
 }
 
@@ -666,6 +752,15 @@ bool Chimera::parse_cpp_type(const QByteArray &type)
 // convert exactly as requested.
 bool Chimera::fromPyObject(PyObject *py, void *cpp) const
 {
+    // Let any registered convertors have a go first.
+    for (int i = 0; i < _registered_QVariant_data_convertors.count(); ++i)
+    {
+        bool ok;
+
+        if (_registered_QVariant_data_convertors.at(i)(py, cpp, _metatype, &ok))
+            return ok;
+    }
+
     int iserr = 0;
 
     PyErr_Clear();
@@ -761,21 +856,12 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
 
     case QMetaType::QVariantList:
         {
-            QList<QObject *> qlo;
+            QVariantList ql;
 
-            if (to_QList_QObject(py, qlo))
-            {
-                *reinterpret_cast<QList<QObject *> *>(cpp) = qlo;
-            }
+            if (to_QVariantList(py, ql))
+                *reinterpret_cast<QVariantList *>(cpp) = ql;
             else
-            {
-                QVariantList ql;
-
-                if (to_QVariantList(py, ql))
-                    *reinterpret_cast<QVariantList *>(cpp) = ql;
-                else
-                    iserr = 1;
-            }
+                iserr = 1;
 
             break;
         }
@@ -903,6 +989,15 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
         return true;
     }
 
+    // Let any registered convertors have a go first.
+    for (int i = 0; i < _registered_QVariant_convertors.count(); ++i)
+    {
+        bool ok;
+
+        if (_registered_QVariant_convertors.at(i)(py, var, &ok))
+            return ok;
+    }
+
     int iserr = 0, value_class_state;
     void *ptr_class, *value_class = 0;
 
@@ -1073,26 +1168,16 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
 
     case QMetaType::QVariantList:
         {
-            QList<QObject *> qlo;
+            QVariantList ql;
 
-            if (to_QList_QObject(py, qlo))
+            if (to_QVariantList(py, ql))
             {
-                *var = QVariant(QList_QObject_metatype(), &qlo);
+                *var = QVariant(ql);
                 metatype_used = QMetaType::Void;
             }
             else
             {
-                QVariantList ql;
-
-                if (to_QVariantList(py, ql))
-                {
-                    *var = QVariant(ql);
-                    metatype_used = QMetaType::Void;
-                }
-                else
-                {
-                    iserr = 1;
-                }
+                iserr = 1;
             }
 
             break;
@@ -1250,7 +1335,7 @@ PyObject *Chimera::toPyObject(const QVariant &var) const
         {
             PyErr_Format(PyExc_TypeError,
                     "unable to convert a QVariant of type %d to a QMetaType of type %d",
-                    _metatype, var.userType());
+                    var.userType(), _metatype);
 
             return 0;
         }
@@ -1275,32 +1360,13 @@ PyObject *Chimera::toPyObject(const QVariant &var) const
         }
     }
 
-    // Handle QList<QObject *>.
-    int lo_metatype = QList_QObject_metatype();
-
-    if (lo_metatype != 0 && lo_metatype == var.userType())
+    // Let any registered convertors have a go first.
+    for (int i = 0; i < _registered_PyObject_convertors.count(); ++i)
     {
-        const QList<QObject *> *cpp = reinterpret_cast<const QList<QObject *> *>(var.data());
+        PyObject *py;
 
-        PyObject *py_list = PyList_New(cpp->count());
-
-        if (!py_list)
-            return 0;
-
-        for (int i = 0; i < cpp->count(); ++i)
-        {
-            PyObject *obj = sipConvertFromType(cpp->at(i), sipType_QObject, 0);
-
-            if (!obj)
-            {
-                Py_DECREF(py_list);
-                return 0;
-            }
-
-            PyList_SET_ITEM(py_list, i, obj);
-        }
-
-        return py_list;
+        if (_registered_PyObject_convertors.at(i)(&var, &py))
+            return py;
     }
 
     return toPyObject(const_cast<void *>(var.data()));
@@ -1365,8 +1431,16 @@ PyObject *Chimera::toPyObject(void *cpp) const
         break;
 
     case QMetaType::UInt:
-        py = SIPLong_FromLong(*reinterpret_cast<unsigned int *>(cpp));
-        break;
+        {
+            long ui = *reinterpret_cast<unsigned int *>(cpp);
+
+            if (ui < 0)
+                py = PyLong_FromUnsignedLong((unsigned long)ui);
+            else
+                py = SIPLong_FromLong(ui);
+
+            break;
+        }
 
     case QMetaType::Double:
         py = PyFloat_FromDouble(*reinterpret_cast<double *>(cpp));
@@ -1576,6 +1650,10 @@ PyObject *Chimera::toAnyPyObject(const QVariant &var)
         return Py_None;
     }
 
+    // Handle null variants for the v2 API.
+    if (var.isNull() && sipIsAPIEnabled("QVariant", 2, 0))
+        return qpycore_qpynullvariant_FromQVariant(var);
+
     const char *type_name = var.typeName();
     const sipTypeDef *td = sipFindType(type_name);
 
@@ -1604,52 +1682,6 @@ QVariant Chimera::keep_as_pyobject(PyObject *py)
 }
 
 
-// Get the metatype for QList<QObject *> or 0 if it hasn't been registered.
-int Chimera::QList_QObject_metatype()
-{
-    static int lo_metatype = 0;
-
-    // We look each time until we find it because QtDeclarative could register
-    // it at any time.
-    if (lo_metatype == 0)
-        lo_metatype = QMetaType::type("QList<QObject*>");
-
-    return lo_metatype;
-}
-
-
-// Convert a Python list object to a QList<QObject*> and return true if there
-// was no error.
-bool Chimera::to_QList_QObject(PyObject *py, QList<QObject *>&cpp) const
-{
-    // We only try to convert if QList<QObject *> is a registered metatype.
-    if (QList_QObject_metatype() == 0)
-        return false;
-
-    Q_ASSERT(PyList_CheckExact(py));
-
-    for (SIP_SSIZE_T i = 0; i < PyList_GET_SIZE(py); ++i)
-    {
-        PyObject *val_obj = PyList_GET_ITEM(py, i);
-
-        if (!val_obj)
-            return false;
-
-        int iserr = 0;
-
-        QObject *val = reinterpret_cast<QObject *>(sipForceConvertToType(
-                val_obj, sipType_QObject, 0, SIP_NO_CONVERTORS, 0, &iserr));
-
-        if (iserr)
-            return false;
-
-        cpp.append(val);
-    }
-
-    return true;
-}
-
-
 // Convert a Python list object to a QVariantList and return true if there was
 // no error.
 bool Chimera::to_QVariantList(PyObject *py, QVariantList &cpp) const
