@@ -1,6 +1,6 @@
 // This contains the implementation of the pyqtBoundSignal type.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -160,7 +155,7 @@ PyTypeObject qpycore_pyqtBoundSignal_Type = {
     0,                      /* tp_dictoffset */
     0,                      /* tp_init */
     0,                      /* tp_alloc */
-    0,                      /* tp_new */
+    PyType_GenericNew,      /* tp_new */
     0,                      /* tp_free */
     0,                      /* tp_is_gc */
     0,                      /* tp_bases */
@@ -738,20 +733,26 @@ static QByteArray slot_signature(Chimera::Signature *signal,
 }
 
 
-// Check that an object is a bound signal and return the bound QObject and the
-// signal signature.
-bool qpycore_pyqtsignal_get_parts(PyObject *sig_obj, QObject **qtx,
-        const char **sig)
+// Check that an object is a signal and return the bound QObject (if requested)
+// and the signal signature.
+const char *qpycore_pyqtsignal_get_parts(PyObject *sig_obj, QObject **qtx)
 {
-    qpycore_pyqtBoundSignal *bs;
+    if (PyObject_TypeCheck(sig_obj, &qpycore_pyqtBoundSignal_Type))
+    {
+        qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)sig_obj;
 
-    if (!PyObject_TypeCheck(sig_obj, &qpycore_pyqtBoundSignal_Type))
-        return false;
+        if (qtx)
+            *qtx = bs->bound_qobject;
 
-    bs = (qpycore_pyqtBoundSignal *)sig_obj;
+        return bs->bound_overload->signature.constData();
+    }
 
-    *qtx = bs->bound_qobject;
-    *sig = bs->bound_overload->signature.constData();
+    if (PyObject_TypeCheck(sig_obj, &qpycore_pyqtSignal_Type) && !qtx)
+    {
+        qpycore_pyqtSignal *ps = (qpycore_pyqtSignal *)sig_obj;
 
-    return true;
+        return ps->overloads->at(ps->master_index)->signature.constData();
+    }
+
+    return 0;
 }
