@@ -60,6 +60,7 @@
 #endif
 
 #include "qpycore_chimera.h"
+#include "qpycore_classinfo.h"
 #include "qpycore_misc.h"
 #include "qpycore_pyqtproperty.h"
 #include "qpycore_pyqtsignal.h"
@@ -177,6 +178,9 @@ static int create_dynamic_metaobject(pyqtWrapperType *pyqt_wt)
     PyTypeObject *pytype = (PyTypeObject *)pyqt_wt;
     qpycore_metaobject *qo = new qpycore_metaobject;
 
+    // Get any class info.
+    QList<ClassInfo> class_info_list = qpycore_get_class_info_list();
+
     // Get the super-type's meta-object.
     qo->mo.d.superdata = get_qmetaobject((pyqtWrapperType *)pytype->tp_base);
 
@@ -369,10 +373,12 @@ static int create_dynamic_metaobject(pyqtWrapperType *pyqt_wt)
     const int header_size = 10;
 #endif
 
-    int data_len = header_size + qo->nr_signals * 5 + qo->pslots.count() * 5 + 
+    int data_len = header_size + class_info_list.count() * 2 +
+            qo->nr_signals * 5 + qo->pslots.count() * 5 + 
             pprops.count() * (has_notify_signal ? 4 : 3) + 1;
     uint *data = new uint[data_len];
-    int g_offset = header_size;
+    int i_offset = header_size;
+    int g_offset = i_offset + class_info_list.count() * 2;
     int s_offset = g_offset + qo->nr_signals * 5;
     int p_offset = s_offset + qo->pslots.count() * 5;
     int n_offset = p_offset + pprops.count() * 3;
@@ -384,6 +390,26 @@ static int create_dynamic_metaobject(pyqtWrapperType *pyqt_wt)
     // The revision number.
     data[0] = revision;
 
+    // Set up any class information.
+    if (class_info_list.count() > 0)
+    {
+        data[2] = class_info_list.count();
+        data[3] = i_offset;
+
+        for (int i = 0; i < class_info_list.count(); ++i)
+        {
+            ClassInfo ci = class_info_list.at(i);
+
+            data[i_offset + (i * 2) + 0] = qo->str_data.size();
+            qo->str_data.append(ci.first.constData());
+            qo->str_data.append('\0');
+
+            data[i_offset + (i * 2) + 1] = qo->str_data.size();
+            qo->str_data.append(ci.second.constData());
+            qo->str_data.append('\0');
+        }
+    }
+
     // Set up the methods count and offset.
     if (qo->nr_signals || qo->pslots.count())
     {
